from setuptools import setup, find_packages

setup(
    name='hivecraft',
    version='0.3.2',
    packages=find_packages(),
    install_requires=[
    ],
    author='Éric PHILIPPE',
    author_email='ericphlpp@proton.me',
    description='Hivecraft is a tool to zip a folder with .alghive extension.',
)